@@ -1021,14 +1021,16 @@ class Client:
 
     # Task Convert
 
-    def export_coco(self, tasks: list, output_dir: str = os.path.join("output", "coco")) -> None:
+    def export_coco(self, tasks: list, annotations: list = [], output_dir: str = os.path.join("output", "coco")) -> None:
         """
         Convert tasks to COCO format and export as a file.
+        If you pass annotations, you can export Pose Estimation type annotations.
 
         tasks is a list of tasks. (Required)
+        annotations is a list of annotations. (Optional)
         output_dir is output directory(default: output/coco). (Optional)
         """
-        coco = converters.to_coco(tasks)
+        coco = converters.to_coco(tasks, annotations)
         os.makedirs(output_dir, exist_ok=True)
         file_path = os.path.join(output_dir, "annotations.json")
         with open(file_path, 'w') as f:
@@ -1276,6 +1278,7 @@ class Client:
         value: str,
         title: str,
         color: str = None,
+        order: int = None,
         attributes: list = []
     ) -> str:
         """
@@ -1297,6 +1300,8 @@ class Client:
         }
         if color:
             payload["color"] = color
+        if order:
+            payload["order"] = order
         if attributes:
             payload["attributes"] = attributes
         return self.api.post_request(endpoint, payload=payload)
@@ -1322,6 +1327,7 @@ class Client:
         value: str = None,
         title: str = None,
         color: str = None,
+        order: int = None,
         attributes: list = []
     ) -> str:
         """
@@ -1341,6 +1347,8 @@ class Client:
             payload["title"] = title
         if color:
             payload["color"] = color
+        if order:
+            payload["order"] = order
         if attributes:
             payload["attributes"] = attributes
         return self.api.put_request(endpoint, payload=payload)
